import { Fragment, Teleport, computed, createStaticVNode, createVNode, defineComponent, getCurrentInstance, h, nextTick, onMounted, ref, toRaw, watch } from "vue";
import { debounce } from "perfect-debounce";
import { hash } from "ohash";
import { appendResponseHeader } from "h3";
import { useHead } from "@unhead/vue";
import { randomUUID } from "uncrypto";
import { joinURL, withQuery } from "ufo";
import { join } from "pathe";
import { useNuxtApp, useRuntimeConfig } from "../nuxt.js";
import { prerenderRoutes, useRequestEvent } from "../composables/ssr.js";
import { getFragmentHTML, getSlotProps } from "./utils.js";
import { remoteComponentIslands, selectiveClient } from "#build/nuxt.config.mjs";
const pKey = "_islandPromises";
const SSR_UID_RE = /nuxt-ssr-component-uid="([^"]*)"/;
const UID_ATTR = /nuxt-ssr-component-uid(="([^"]*)")?/;
const SLOTNAME_RE = /nuxt-ssr-slot-name="([^"]*)"/g;
const SLOT_FALLBACK_RE = /<div nuxt-slot-fallback-start="([^"]*)"[^>]*><\/div>(((?!<div nuxt-slot-fallback-end[^>]*>)[\s\S])*)<div nuxt-slot-fallback-end[^>]*><\/div>/g;
let id = 0;
const getId = import.meta.client ? () => (id++).toString() : randomUUID;
const components = import.meta.client ? /* @__PURE__ */ new Map() : void 0;
async function loadComponents(source = "/", paths) {
  const promises = [];
  for (const component in paths) {
    if (!components.has(component)) {
      promises.push((async () => {
        const chunkSource = join(source, paths[component]);
        const c = await import(
          /* @vite-ignore */
          chunkSource
        ).then((m) => m.default || m);
        components.set(component, c);
      })());
    }
  }
  await Promise.all(promises);
}
function emptyPayload() {
  return {
    chunks: {},
    props: {},
    teleports: {}
  };
}
export default defineComponent({
  name: "NuxtIsland",
  props: {
    name: {
      type: String,
      required: true
    },
    lazy: Boolean,
    props: {
      type: Object,
      default: () => void 0
    },
    context: {
      type: Object,
      default: () => ({})
    },
    source: {
      type: String,
      default: () => void 0
    },
    dangerouslyLoadClientComponents: {
      type: Boolean,
      default: false
    }
  },
  async setup(props, { slots, expose }) {
    const key = ref(0);
    const canLoadClientComponent = computed(() => selectiveClient && (props.dangerouslyLoadClientComponents || !props.source));
    const error = ref(null);
    const config = useRuntimeConfig();
    const nuxtApp = useNuxtApp();
    const filteredProps = computed(() => props.props ? Object.fromEntries(Object.entries(props.props).filter(([key2]) => !key2.startsWith("data-v-"))) : {});
    const hashId = computed(() => hash([props.name, filteredProps.value, props.context, props.source]));
    const instance = getCurrentInstance();
    const event = useRequestEvent();
    const eventFetch = import.meta.server ? event.fetch : import.meta.dev ? $fetch.raw : globalThis.fetch;
    const mounted = ref(false);
    onMounted(() => {
      mounted.value = true;
    });
    function setPayload(key2, result) {
      nuxtApp.payload.data[key2] = {
        __nuxt_island: {
          key: key2,
          ...import.meta.server && import.meta.prerender ? {} : { params: { ...props.context, props: props.props ? JSON.stringify(props.props) : void 0 } },
          result: {
            chunks: result.chunks,
            props: result.props,
            teleports: result.teleports
          }
        },
        ...result
      };
    }
    const rawPayload = nuxtApp.isHydrating ? toRaw(nuxtApp.payload.data)?.[`${props.name}_${hashId.value}`] ?? emptyPayload() : emptyPayload();
    const nonReactivePayload = {
      chunks: rawPayload.chunks,
      props: rawPayload.props,
      teleports: rawPayload.teleports
    };
    const ssrHTML = ref("");
    if (import.meta.client) {
      ssrHTML.value = getFragmentHTML(instance.vnode?.el ?? null, true)?.join("") || "";
    }
    const slotProps = computed(() => getSlotProps(ssrHTML.value));
    const uid = ref(ssrHTML.value.match(SSR_UID_RE)?.[1] ?? getId());
    const availableSlots = computed(() => [...ssrHTML.value.matchAll(SLOTNAME_RE)].map((m) => m[1]));
    const html = computed(() => {
      const currentSlots = Object.keys(slots);
      let html2 = ssrHTML.value;
      if (import.meta.client && !canLoadClientComponent.value) {
        for (const [key2, value] of Object.entries(nonReactivePayload.teleports || {})) {
          html2 = html2.replace(new RegExp(`<div [^>]*nuxt-ssr-client="${key2}"[^>]*>`), (full) => {
            return full + value;
          });
        }
      }
      return html2.replace(SLOT_FALLBACK_RE, (full, slotName, content) => {
        if (currentSlots.includes(slotName)) {
          return "";
        }
        return content;
      });
    });
    function setUid() {
      uid.value = ssrHTML.value.match(SSR_UID_RE)?.[1] ?? getId();
    }
    const cHead = ref({ link: [], style: [] });
    useHead(cHead);
    async function _fetchComponent(force = false) {
      const key2 = `${props.name}_${hashId.value}`;
      if (nuxtApp.payload.data[key2]?.html && !force) {
        return nuxtApp.payload.data[key2];
      }
      const url = remoteComponentIslands && props.source ? new URL(`/__nuxt_island/${key2}.json`, props.source).href : `/__nuxt_island/${key2}.json`;
      if (import.meta.server && import.meta.prerender) {
        nuxtApp.runWithContext(() => prerenderRoutes(url));
      }
      const r = await eventFetch(withQuery(import.meta.dev && import.meta.client || props.source ? url : joinURL(config.app.baseURL ?? "", url), {
        ...props.context,
        props: props.props ? JSON.stringify(props.props) : void 0
      }));
      const result = import.meta.server || !import.meta.dev ? await r.json() : r._data;
      if (import.meta.server && import.meta.prerender) {
        const hints = r.headers.get("x-nitro-prerender");
        if (hints) {
          appendResponseHeader(event, "x-nitro-prerender", hints);
        }
      }
      setPayload(key2, result);
      return result;
    }
    async function fetchComponent(force = false) {
      nuxtApp[pKey] = nuxtApp[pKey] || {};
      if (!nuxtApp[pKey][uid.value]) {
        nuxtApp[pKey][uid.value] = _fetchComponent(force).finally(() => {
          delete nuxtApp[pKey][uid.value];
        });
      }
      try {
        const res = await nuxtApp[pKey][uid.value];
        cHead.value.link = res.head.link;
        cHead.value.style = res.head.style;
        ssrHTML.value = res.html.replace(UID_ATTR, () => {
          return `nuxt-ssr-component-uid="${getId()}"`;
        });
        key.value++;
        error.value = null;
        if (selectiveClient && import.meta.client) {
          if (canLoadClientComponent.value && res.chunks) {
            await loadComponents(props.source, res.chunks);
          }
          nonReactivePayload.props = res.props;
        }
        nonReactivePayload.teleports = res.teleports;
        nonReactivePayload.chunks = res.chunks;
        if (import.meta.client) {
          await nextTick();
        }
        setUid();
      } catch (e) {
        error.value = e;
      }
    }
    expose({
      refresh: () => fetchComponent(true)
    });
    if (import.meta.hot) {
      import.meta.hot.on(`nuxt-server-component:${props.name}`, () => {
        fetchComponent(true);
      });
    }
    if (import.meta.client) {
      watch(props, debounce(() => fetchComponent(), 100), { deep: true });
    }
    if (import.meta.client && !nuxtApp.isHydrating && props.lazy) {
      fetchComponent();
    } else if (import.meta.server || !nuxtApp.isHydrating || !nuxtApp.payload.serverRendered) {
      await fetchComponent();
    } else if (selectiveClient && canLoadClientComponent.value && nonReactivePayload.chunks) {
      await loadComponents(props.source, nonReactivePayload.chunks);
    }
    return () => {
      if (!html.value || error.value) {
        return [slots.fallback?.({ error: error.value }) ?? createVNode("div")];
      }
      const nodes = [createVNode(Fragment, {
        key: key.value
      }, [h(createStaticVNode(html.value || "<div></div>", 1))])];
      if (uid.value && (mounted.value || nuxtApp.isHydrating || import.meta.server) && html.value) {
        for (const slot in slots) {
          if (availableSlots.value.includes(slot)) {
            nodes.push(createVNode(Teleport, { to: import.meta.client ? `[nuxt-ssr-component-uid='${uid.value}'] [nuxt-ssr-slot-name='${slot}']` : `uid=${uid.value};slot=${slot}` }, {
              default: () => (slotProps.value[slot] ?? [void 0]).map((data) => slots[slot]?.(data))
            }));
          }
        }
        if (import.meta.server) {
          for (const [id2, html2] of Object.entries(nonReactivePayload.teleports ?? {})) {
            nodes.push(createVNode(Teleport, { to: `uid=${uid.value};client=${id2}` }, {
              default: () => [createStaticVNode(html2, 1)]
            }));
          }
        }
        if (selectiveClient && import.meta.client && canLoadClientComponent.value) {
          for (const [id2, props2] of Object.entries(nonReactivePayload.props ?? {})) {
            const component = components.get(id2.split("-")[0]);
            const vnode = createVNode(Teleport, { to: `[nuxt-ssr-component-uid='${uid.value}'] [nuxt-ssr-client="${id2}"]` }, {
              default: () => {
                return [h(component, props2)];
              }
            });
            nodes.push(vnode);
          }
        }
      }
      return nodes;
    };
  }
});
